package com.qihoo.security.services;

import com.qihoo.security.services.IScanCallback;
import com.qihoo.security.services.ScanResult;
import com.qihoo.security.services.DeepScanItem;

/**
 * QVS SDK 的客户端调用接口。客户端通过调用下面的 AIDL 接口与 QVS SDK 的服务进行交互。
 *
 * @author zhangxu
 */
interface IDeepScan
{
	/** 
	 * 获取引擎当前版本，例如 1001。本方法是常数查询，在引擎初始化前就可以调用。
	 * @return 返回内部版本号，比如1001。
	 */
	int getVersion();

	/**
	 * 全局初始化，包括加载配置文件、启动引擎进程等。在开始扫描前，必须调用此接口。
	 * @return S_OK：成功,  E_XXX：失败
	 * @see #uninit()
	 */
	int init();

	/**
	 * 释放引擎占用资源，退出引擎进程。调用 uninit() 方法之后，必须要重新调用 init() 方法才能重新开始扫描。
	 * @return S_OK：成功,  E_XXX：失败
	 * @see #init()
	 */
	int uninit();

	/**
	 * 设置全局配置，在引擎创建之前设置，比如设置网络参数、设置日志文件目录等。和 setOption() 的区别：setOption() 是用于控制引擎的行为；setGlobalOption 是设置引擎的数据。
	 *  
	 * @param key 配置key
	 * @param value 参数值
	 * @return  S_OK：成功,  E_XXX：失败
	 */
	int setGlobalOption(in String key, in String value);

	/**
	 * 注册回调接口。扫描的进度和结果会通过此接口通知。本方法可以在 init() 之前调用，也可以在之后调用，如果是在 init() 之后调用，那么可能收不到 IScanCallback 的 onReady() 事件。
	 * @param callback 回调接口
	 * @return 成功则返回 true，如果已经调用过 uninit() 了，那么会注册失败，返回 false.
	 * @see IScanCallback 
	 */
	boolean registerCallback(in IScanCallback callback);
	
	/**
	 * 取消对回调接口的注册。当不再使用杀毒引擎时，务必取消接口注册，否则引擎仍然会继续调用回调接口。
	 * @param callback 回调接口
	 * @see IScanCallback
	 */
	void unregisterCallback(in IScanCallback callback);

	/**
	 * 获取引擎配置参数
	 * @param key 参数名称，用点分路径表示，比如：360.product，或者 generic.setloginitinfo 等
	 * @return 配置的当前值，用字符串表示 
	 */
	String getOption(in String key);

	/**
	 * 设置引擎配置参数
	 * @param key 参数名称，用点分路径表示，比如：360.product，或者 generic.setloginitinfo 等
	 * @param value 参数值
	 * @return 该配置改变前的值，如果之前没有设置过这个参数，返回空串
	 */
	String setOption(in String key, in String value);

	/**
	 * 启动扫描指定的项目，如已安装应用，SD卡文件，启动项等等。本方法只启动扫描，扫描的结果通过 registerCallback() 注册的回调接口通知。
	 * @param scanList 要扫描的项目的列表。例如，如果要进行“全盘扫描”，即同时扫描已安装应用和SD卡上的安装包，那么列表就包括这两项。
	 * @return 正常情况返回 S_OK，如果遇到错误，可能的返回值：
	 * <ul>
	 * <li>{@link HRESULT#E_UNEXPECTED} 引擎尚未初始化</li>
	 * <li>{@link HRESULT#E_PENDING} 正在扫描，需要等待扫描结束之后才能调用</li>
	 * </ul>
	 */
	int scan(in List<DeepScanItem> scanList);
	
	/**
	 * 扫描一个已安装的应用，用于安装监控等需求。注意：调用此方法就会触发一次云查杀请求，效率比较低，所以如果是批量扫描，不要调用本方法。
	 * @param packageName 要扫描的包名。该包必须已经安装在系统中。
	 * @return 如果要扫描的包存在，则返回 S_OK. 如果该包不存在，则返回 HRESULT#E_INVALIDARG.
	 */
	int scanPackage(in String packageName);
	
	/**
	 * 扫描单个文件，用于安装监控等需求。注意：调用此方法就会触发一次云查杀请求，效率比较低，所以如果是批量扫描，不要调用本方法。
	 * @param filename 要扫描的文件名。
	 * @return 正常情况返回 S_OK. 如果文件不存在，则返回 HRESULT#E_INVALIDARG.
	 */
	int scanFile(in String filename);

	/**
	 * 暂停扫描，调用本方法后引擎处于阻塞状态，只有调用 resume() 之后才恢复扫描。
	 * @return S_OK：成功, E_XXX：失败
	 * @see #resume()
	 */
	int pause();

	/** 
	 * 与 pause() 配套，pause() 之后用此方法恢复扫描。
	 * @return S_OK：成功, E_XXX：失败
	 * @see #pause()
	 */
	int resume();

	/**
	 * 停止扫描。本方法是异步的，调用之后，引擎可能还会继续运行一小段时间，最终停止后，通过 IScanCallback.onStop() 通知调用者。停止之后，必须调用 reset() 之后才能重新启动扫描。
	 * @return S_OK：成功, E_XXX：失败
	 * @see IScanCallback#onStop()
	 * @see #reset()
	 */
	int stop();
	
	/**
	 * 停止扫描之后，调用本方法复位之后才能重新启动扫描。注意在调用 stop() 之后，如果在收到 IScanCallback.onStop() 事件之前就调用本方法，可能发生不可预知的错误。
	 * @return S_OK：成功, E_XXX：失败
	 * @see #stop()
	 */
	int reset();

	/**
	 * 修复扫描出的木马，例如删除木马文件以及感染的文件等。
	 * @param resultItem
	 * @return  S_OK：成功,  E_XXX：失败
	 */
	int fix(in List<ScanResult> resultItem);
}
