package com.qihoo.security.services;

import com.qihoo.security.services.IRestoreManager;
import com.qihoo.security.services.ScanResult;
import com.qihoo.security.services.IScanCallback;

/**
 * 杀毒引擎模块的抽象接口。例如 BitDefender 引擎，本地 AVE 引擎，云查询引擎，QVM引擎等，都用此接口进行封装。
 * <p>
 * QVS SDK的整体架构是：客户端 app 通过 DeepScanService 与 QVS SDK 交互，DeepScanService 负责调度各杀毒引
 * 擎模块，而各杀毒引擎模块封装在 ScanEngineService 服务这个容器中，运行在一个独立进程内，这样当杀毒引擎模
 * 块故障时，不会影响整体 SDK。ScanEngineService 与 DeepScanService 之间通过 AIDL 接口进行交互。
 * </p>
 *
 *  @author kongqinglong
 */
interface IEngineBase
{
	/**
	 * 引擎预加载，通常是加载病毒库。
	 * 有的引擎加载病毒库比较耗时，提前加载减少用户点击扫描时卡的现象，提高用户体验。
	 * 是否采用阻塞式加载由实现者决定。
	 * 
	 * @param  无
	 * @return 成功返回S_OK, 失败返回E_XXX系列
	 */
	int preload(in int engineId);

	/**
	 * 卸载病毒库，释放内存
	 * 
	 * @param  无
	 * @return 成功返回S_OK, 失败返回E_XXX系列
	 */
	int unload(in int engineId);

    /**
	 * 引擎初始化，调用引擎之前需要先调用本接口。
	 *
	 * @return 成功返回S_OK, 失败返回E_XXX系列
	 */
    int init(in int engineId);

	/**
	 * 当不再需要调用引擎时，调用此接口释放资源。注意：可能会有好几个线程同时调用引擎模块的情况，其中一个线程调用 uninit() 只是减小引擎的计数，引擎本身仍然会处于运行状态。
	 * 
	 * @return 成功返回S_OK, 失败返回E_XXX系列
	 */
    int uninit(in int engineId);

	/**
	 * 引擎重置。取消所有正在扫描的任务，将引擎的状态变为刚完成 preload() 和 init() 之后的状态。
	 * @return 成功返回S_OK, 失败返回E_XXX系列
	 */
    int reset(in int engineId);

	/**
	 * 通用的设置引擎属性的方法。引擎的属性都用key-value型来设置。
	 *
	 * @param  name  属性名称
	 * @param  value 属性值
	 *
	 * @return 成功返回S_OK, 失败返回E_XXX系列
	 */
	int setOption(in int engineId, in String name, in String value); 

    /**
	 *  扫描项预处理，调用 scan 接口前，可以先调用本方法来检查一下是否能立即给出结果。
	 *  <p>本方法中的操作*务必*是轻量级的，否则会阻塞主控流程
	 *	<ul>
	 *  <li>对于轻量的操作马上能给出结果的，直接返回 ScanResult 结果对象；</li>
	 *  <li>非轻量的扫描操作或累积多项后才处理的，本方法返回 null， 主控内部的任务调度会调用引擎的scan接口</li>
	 *	</ul>
	 *	</p>
	 *  @param item  待扫描项
	 *
	 *  @return 如果能立即返回结果，那么返回 S_OK。如果暂时不能返回结果，那么返回 S_FALSE。如果出错，返回 E_XXX 系列。
	 */ 
    int preScan(in int engineId, inout ScanResult item);

	/**
	 * 执行扫描操作。本方法是阻塞的，所以需要在一个线程内执行。
	 *	 
	 * @param scanList 要扫描的列表。扫描时会更新该列表内的 ScanResult 对象。
	 * @return 扫描正常结束返回 S_OK，否则返回 E_XXX 系列。
	 */
    int scan(in int engineId, inout List<ScanResult> scanList);

	/**
	 *  将要处理的文件备份到隔离区（预留）
	 *  
	 *  @param item 待备份项
	 *  @param restoreManager 隔离区管理器
	 */
	int backup(in int engineId, in ScanResult item, in IRestoreManager restoreManager);

    /**
	 *  处理扫描出的文件，如修复、删除等等。
	 *  @param item : 待处理项
	 *
	 *  @return 处理结果
	 *  @see    ConstFixResult
	 */
    int process(in int engineId, in ScanResult item);

    /**
	 * 停止处理
	 * 异步接口配合getState()判断引擎是否结束
	 * @see #getState()
	 */
    int stop(in int engineId);
    
    /** 扫描完成之后，调用本方法进行一些清理的工作，例如保存缓存数据、删除临时文件等。*/
    int finish(in int engineId);

    /**
	 * 返回引擎的当前状态，如“运行中”等
	 *
	 * @see ConstEngineState
	 * @param  无
	 * @return 当前引擎的状态
	 */
    int getState(in int engineId) ;
        
	/** 返回当前激活的引擎所对应的名称 */
	String getName(in int engineId);
	
	/** 扫描结束后进行清理 */
	void cleanup(in ScanResult scanResult);
	
	/**
	 * 查询全局统计数据。
	 * @param key 要查询的项目。如果 key 为空，那么返回所有的统计。
	 * @return JSON格式的统计结果。
	 */
	String getGlobalStatistics(String key);
	
	/**
	 * 从EngineService内部创建出一个指定ID的独立子引擎，由本接口创建的子引擎与EngineService自身默认创建的那套引擎无任何联系
	 * @param engineId 要创建的子引擎ID
	 * @return 子引擎的IEngineBase接口
	*/
	IBinder createSingleSubEngine(in int engineId);
	
	int update(in int engineId, in int cmd, in String data);
}

